#define B_Y			(1<<0)
#define B_B			(1<<1)
#define B_A			(1<<2)
#define B_X			(1<<3)

#define OFF 0
#define RECORDING 1
#define REPLAYING 2
#define ATRACTING 3
#define REVIEWING 4

#define BLUE_P			(1<<0)
#define RED_P			(1<<1)
#define YELLOW_P		(1<<2)
#define GREEN_P			(1<<3)
#define TMPLAY		(1<<4)

static unsigned int replays_counter=0;


void store_controller(global *g_point, cont_cond_t source)
	{
	c_frame output;

	output.rtrig=source.rtrig;
	output.ltrig=source.ltrig;
	output.joyx=source.joyx;
	output.joyy=source.joyy;

	output.buttons=0;

	if (!(source.buttons & CONT_X ))
		output.buttons+=B_X;

	if (!(source.buttons & CONT_Y ))
		output.buttons+=B_Y;

	if (!(source.buttons & CONT_A ))
		output.buttons+=B_A;

	if (!(source.buttons & CONT_B))
		output.buttons+=B_B;

	uint tmp=g_point->r_control_box.current_frame++;

	g_point->r_control_box.replay->data[tmp]=output;
	}

cont_cond_t recal_controller(global *g_point)
	{
	uint tmp=g_point->r_control_box.current_frame++;

	c_frame source=g_point->r_control_box.replay->data[tmp];

	cont_cond_t output;

	output.buttons=~0;

	output.rtrig=source.rtrig;
	output.ltrig=source.ltrig;
	output.joyx=source.joyx;
	output.joyy=source.joyy;

	if (source.buttons & B_X)
		output.buttons-=CONT_X;

	if (source.buttons & B_Y)
		output.buttons-=CONT_Y;

	if (source.buttons & B_A)
		output.buttons-=CONT_A;

	if (source.buttons & B_B)
		output.buttons-=CONT_B;

	return output;
	}

void start_replaying(int status, global *g_point)
	{
	g_point->r_control_box.status=status;
	initialize_players(g_point);
	g_point->in_menu=FALSE;
	g_point->r_control_box.current_frame=0;
	}

void start_recording(global *g_point)
	{
	memset ( g_point->r_control_box.replay->data, 0, sizeof(mega3)-sizeof(replay_header));

	g_point->r_control_box.status=RECORDING;
	g_point->in_menu=FALSE;

	initialize_players(g_point);

	int i=0;

	player *active=g_point->first_player;

	g_point->r_control_box.replay->active_players=0;

	while(i<g_point->number_of_players)
		{
		switch (active->controller)
		{
		case 32: g_point->r_control_box.replay->active_players+=BLUE_P;
		break;

		case 96: g_point->r_control_box.replay->active_players+=RED_P;
		break;

		case 160: g_point->r_control_box.replay->active_players+=GREEN_P;
		break;

		case 224: g_point->r_control_box.replay->active_players+=YELLOW_P;
		break;
		}
		active=active->next;
		i++;
		}

	g_point->r_control_box.replay->starting_position.X=-16.0f;
	g_point->r_control_box.replay->starting_position.Y=16.0f;
	g_point->r_control_box.replay->starting_orientation=PI*0.75;

	g_point->r_control_box.status=RECORDING;
	g_point->r_control_box.replay->length=0;
	g_point->r_control_box.current_frame=0;
	}

void stop_recording(global *g_point)
	{
		if(g_point->r_control_box.status==RECORDING)
			{
			g_point->r_control_box.replay->length=g_point->r_control_box.current_frame;
			g_point->r_control_box.current_frame=0;
			}
	}

void stop_replaying(global *g_point)
	{

	switch (g_point->r_control_box.status)
		{
		case REVIEWING:
			g_point->in_menu=SCOREBOARD;
			g_point->r_control_box.replay_mode=NORMAL_REPLAY;
			break;


		case ATRACTING:
			g_point->in_menu=MAIN_MENU;
			g_point->r_control_box.replay_mode=NORMAL_REPLAY;
			break;

		default:
			g_point->in_menu=MAIN_MENU;
			g_point->r_control_box.replay_mode=NORMAL_REPLAY;
		}
	}

int store_replay(global *g_point)
	{

	file_t	f;

	char filename[512];

	strcpy(filename, g_point->base_dir);

	strcpy((filename + strlen(g_point->base_dir)), "replays/replay0000.dre");

	file_t check=1;

int c=strlen(g_point->base_dir)+strlen("replays/replay0000.dre");

while (check)
	{
	filename[c-5]='0';
	filename[c-6]='0';
	filename[c-7]='0';
	filename[c-8]='0';

	replays_counter++;

	uint thousands, hundreds, tens, ones;

	thousands=replays_counter/1000;
	hundreds=(replays_counter-(1000*thousands))/100;
	tens=(replays_counter-(1000*thousands)-(100*hundreds))/10;
	ones=replays_counter-(1000*thousands)-(100*hundreds)-(10*tens);

	filename[c-8]+=thousands;
	filename[c-7]+=hundreds;
	filename[c-6]+=tens;
	filename[c-5]+=ones;

	check=fs_open((const char *)&filename, O_RDONLY);
	}

	fs_close(check);

	 f = fs_open((const char*)filename, O_WRONLY);
		if (!f) {
			dbglog(DBG_ERROR, "write config: can't open %s \n", filename);
			return -1;
		}

	/* Write the data */
	if (fs_write(f, g_point->r_control_box.replay, g_point->r_control_box.replay->length*sizeof(c_frame)+sizeof(replay_header)) != g_point->r_control_box.replay->length*sizeof(c_frame)+sizeof(replay_header)) {
		printf("store_recording: can't write data to output file '%s'\n", (const char *)&filename);
		fs_close(f);
		return -1;
		}

	fs_close(f);

return 0;
	}

int recal_replay(int number, global *g_point)
	{
	memset ( g_point->r_control_box.replay->data, 0, sizeof(mega3)-sizeof(replay_header));

	int length;
	char filename[512];

	strcpy (filename, g_point->base_dir);

	strcpy ((filename + strlen(g_point->base_dir)), "replays/replay0000.dre");

	uint thousands, hundreds, tens, ones;

	thousands=number/1000;
	hundreds=(number-(1000*thousands))/100;
	tens=(number-(1000*thousands)-(100*hundreds))/10;
	ones=number-(1000*thousands)-(100*hundreds)-(10*tens);

int c=strlen(g_point->base_dir)+strlen("replays/replay0000.dre");


	filename[c-8]+=thousands;
	filename[c-7]+=hundreds;
	filename[c-6]+=tens;
	filename[c-5]+=ones;

	uint fd = fs_open((const char *)&filename, O_RDONLY);
	if (!fd) {
		printf("recal_config: can't open input file '%s'\n", (const char *)&filename);
		return -1;}

	fs_seek(fd, 0, SEEK_END);
	length=fs_tell(fd);

	fs_seek(fd, 0, SEEK_SET);

	fs_read(fd, g_point->r_control_box.replay, length);

	fs_close(fd);

	return 0;
	}
