#define PI	3.141592f
#define PI2 6.283185f
#define PI_2 1.570796f

#define PI_32 0.087266462f
#define PI_16 0.19634954f
#define MENU_DELAY 15

enum Texture_Objects {ZERO=0, ONE, TWO, THREE, FOUR, FIVE, SIX, SEVEN, EIGHT, NINE, TITLE, WHITE, PAUSE, SCOREBOARD, DISCLAIMER, START_BUTTON, BUTTON, ROUNDS_PLAYED, VGI};
#define NUM_TEXTURES 19

enum Menues {MAIN_MENU=1,PAUSE_MENU};


/*offensive types*/
#define SHOT 1
#define EXPLOSION 2
#define ARTI 3
#define ARTI_EXP 4
#define MINE_EXPLOSION 5
#define MINE_PASSIVE_OFFENSIVE 6

/*game rules*/
#define TEAMPLAY 0
#define ENGINE_SOUNDS 1
#define AIR_DENSITY 0.05
#define JOYSTICK_SENSITIVITY 0.0004
#define PLAYER_ACCELERATION 0.00003
#define SHOT_SPEED 0.7f
#define PLAYER_START_HEALTH 270
#define ASSAILANT_EARNS_HEALTH 0.33
#define PARTICLE_SIZE 0.20f
#define TRAIL_LENGTH 64


#define BLUE 1
#define RED 2
#define TRUE 1
#define FALSE 0

#define MIST_A_DECREASE 0.012f
#define MIST_C_DECREASE 0.008f
#define MIST_A_MAX 0.5f

#define SHOT_REPEAT_DELAY 15
#define MIN_PLAYER_SIZE 0.850f
#define MAX_PLAYER_SIZE 3.0f

#define MINE_PULSE_RATE 30
#define MINE_SIZE 0.6f
#define MINE_COUNTDOWN 590
#define MINE_STRENGTH 36
#define MINE_DECREASE_PER_TURN 5

#define COST_SHOT 10.0f
#define COST_MINE 15.0f
#define COST_HEALTH 2.5f
#define COST_JUIZE 1.5f

#define JUIZE_CHARGE_RATE 0.13f






/*replay control defines */
#define NORMAL_REPLAY 0
#define SLOW_MOTION_REPLAY 1
#define	STOP_FRAME_REPLAY 2
#define ADVANCE_ONE_FRAME_REPLAY 3



typedef struct {
	float X;
	float Y;
}vector;


typedef struct {
	int a;
	int b;
	int y;
	int x;
	int d_up;
	int d_down;
	int d_left;
	int d_right;
	int start;
	}buttons;


typedef struct {
	uint controller;			//controller ID
	cont_cond_t *c_input;
	cont_cond_t *o_input;
	int health;
	float juize;
	int engine_sound;
	int shield_sound;
	int morphing;
	int bouncing;
	int knocked_out;
	int collision;				//has it suffered colision this frame?
	int score[5][2];			//sum of scores for all rounds
	int this_round[5][2];		//score for this round
	int type;					//player type
	float size;					//players size
	float speed;				//players current velocity
	float orientation;			//players current orientation
	vector position;			//players current position
	vector velocity;			//velocity for red type
	vector bar_position;		//the various status bars
	float red;					//ship colour
	float green;				//ship colour
	float blue; 				//ship colour
	buttons button;
	void *next;					//pointer to next player, NULL if last in row.
	void *prev;					//pointer to prev player, NULL if first in row
	void *arti;
	int arti_countdown;
	float t_intensity;
	} player;

typedef struct {
	int type;
	player *source;
	vector position;
	float size;
	float orientation;
	float speed;
	int strength;
	int shot_strength;			//ttl in frames
	float red;					//offensive colour
	float green;				//offensive colour
	float blue; 				//offensive colour
	void *next;					//pointer to next offensive, NULL if last in row.
	void *prev;					//pointer to prev offensive, NULL if first in row
/*for use with explosions*/
	int inherited_shot;
	float dispersion;
	float D;
/*for use with arti*/
	float cape_red, cape_green, cape_blue;
	}offensive;


typedef struct {
	int type;
	player *source;
	vector position;
	vector velocity;
	int collision;
	int countdown;
	int pulse;
	float size;
	float strength;
	float red;					//mine colour
	float green;				//mine colour
	float blue; 				//mine colour
	void *next;					//pointer to next mine NULL if last in row.
	void *prev;					//pointer to prev mine, NULL if first in row
	}mine;


typedef struct {
	float red, green, blue, alpha;
	}colour;

typedef struct {
	float intensity;
	vector middle_v, left_v, right_v;
	}trail_element;

typedef struct {
	int newest;
	trail_element *trail;
	}trail_header;

typedef struct
	{
	uint8 rtrig;			/* right trigger */
	uint8 ltrig;			/* left trigger */
	uint8 joyx;			/* joystick X */
	uint8 joyy;			/* joystick Y */
	uint8 buttons;
	}c_frame;

typedef struct
	{
	uint length;
	uint8 replay_version;
	uint8 active_players;
	vector starting_position;
	float starting_orientation;
	c_frame *data;
	}replay_header;


typedef struct
	{
	uint current_frame;
	int status;
	replay_header *replay;
	int replay_mode;
	int frame_counter;
	int stop_controller;
	}replay_control_box;


typedef struct
	{
	char tmp[3*1024*1024];
	}mega3;

typedef struct {
	int in_menu;
	int menu_delay;
	int menu_controller;
	int number_of_players;
	int exit_program;
	int player_start_health;
	int max_shot_strength;
	int mine_strength;
	float shot_health_modifier;
	float assailant_earns;
	float shot_speed;
	float air_d;				//atmosferic density
	float centre_gravity;		//gravital pull towards the centre;
	float joy_response;			//factor for how responsive the joystick is
	float p_acceleration;		//player acceleration
	float particle_size;
	float juize_increase;
	float mine_explode_push;
	float trail_opasity;
	int trail_length;
	int mine_countdown;
	float cost_shot;
	float cost_mine;
	int shot_repeat_delay;
	float cost_health;
	float cost_juize;
	float mist_a_decrease;
	int teamplay;
	int sound_settings;
	char *base_dir;
	float shot_player_push;
	float shot_mine_push;
	float flash_intensity;

	cont_cond_t A;
	cont_cond_t B;
	cont_cond_t C;
	cont_cond_t D;

	cont_cond_t A_old;
	cont_cond_t B_old;
	cont_cond_t C_old;
	cont_cond_t D_old;

	player *PA;
	player *PB;
	player *PC;
	player *PD;

	int passive_controllers;

	uint round_time;
	uint demo_time;
	player *first_player;			//pointer to first player
	player *last_player;
	colour mist;
	offensive *first_offensive;
	offensive *last_offensive;
	offensive *first_explosion;
	offensive *last_explosion;
	mine *first_mine;
	mine *last_mine;
	int scoreboard[5][4];
	int rounds_played;
	GLuint	texture[NUM_TEXTURES];
	int sfx_lastchan;
	float mist_max_a;
	trail_header *trails[4];
	replay_control_box r_control_box;
	}global;

GLvoid ReSizeGLScene();
offensive *create_offensive(player *orign, global *g_point, int type, int sound);
offensive *create_explosion(float red, float green, float blue, global *g_point, vector position, int strength, int type);
void remove_offensive (offensive *obj, global *g_point);
void update_offensive(offensive *current, global *g_point);
void update_offensives(global *g_point);
float shot_player_collision(offensive *current, player *target, global *g_point);
player *create_player (vector position, float orientation,uint32 controller, global *g_point,float red, float green, float blue);
void quit_player (player *quiter, global *g_point);
void update_player(player *original, global *g_point);
void update_players(global *g_point);
float player_player_collision(player *current, player *target, global *g_point);
mine *create_mine(player *orign, global *g_point);
int shot_mine_collision(offensive *current, mine *target, global *g_point);
int mine_mine_collision(mine *current, mine *target, global *g_point);
void remove_mine (mine *obj, global *g_point);

void draw_scoreboard(global *g_point);
void reset_scoreboard(global *g_point);
void scoreboard_add_kill(player *target, player *attacker, global *g_point);
int scoreboard_add_victory(global *g_point);
void play_sfx(int idx, int vol, int pan, global *g_point);
int pos2pan(float position);
void initialize_players(global *g_point);
